#define _GNU_SOURCE
#include "common.h"

#include <inttypes.h>
#include <signal.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

void (*previous_handlers[NSIG])(int);
static void (*sighand)(int);

uint64_t ts_to_uint(struct timespec t) {
  return t.tv_sec * NSEC_PER_SEC + t.tv_nsec;
}
struct timespec uint_to_ts(uint64_t t) {
  struct timespec ts;
  ts.tv_sec = t / NSEC_PER_SEC;
  ts.tv_nsec = t - (ts.tv_sec * NSEC_PER_SEC);
  return ts;
}

void add_ns(struct timespec *t, uint64_t ns) {
  t->tv_nsec += ns;

  while ((unsigned int)t->tv_nsec >= NSEC_PER_SEC) {
    t->tv_sec += 1;
    t->tv_nsec -= NSEC_PER_SEC;
  }
}

uint64_t calcdiff_ns(struct timespec t1, struct timespec t2) {
  uint64_t diff;
  diff = NSEC_PER_SEC * (uint64_t)((int)t1.tv_sec - (int)t2.tv_sec);
  diff += ((int)t1.tv_nsec - (int)t2.tv_nsec);
  return diff;
}

int _max_(int a, int b) { return a > b ? a : b; }
int _min_(int a, int b) { return a < b ? a : b; }

int histogram_min(uint64_t *histogram, int max_value) {
  int ret = max_value;
  for (int i = max_value; i >= 0; i--) ret = histogram[i] ? i : ret;
  return ret;
}
int histogram_max(uint64_t *histogram, int max_value) {
  int ret = 0;
  for (int i = 0; i <= max_value; i++) ret = histogram[i] ? i : ret;
  return ret;
}

static void sighand_wrapper(int sig) {
  // If we get un unexpected signal, report it, if not print the histogram
  if (sig == SIGINT || sig == SIGTERM)
    (*sighand)(sig);  // Will print the histogram
  else
    printf("Uknown signal interrupt: %s (%d)\n", strsignal(sig), sig);

  // Execute the default handler
  if (previous_handlers[sig] == SIG_DFL) {
    signal(sig, SIG_DFL);
    raise(sig);
  } else if (previous_handlers[sig] == SIG_IGN) {
    return;
  } else {
    (*previous_handlers[sig])(sig);
  }
}

void init_signals(void (*_sighand)(int)) {
  sighand = _sighand;

  for (int i = 0; i < NSIG; i++) signal(i, sighand_wrapper);
}

